@@ -37,6 +37,8 @@
 
 #include "set-timezone.h"
 
+#define CACHE_VALIDITY_SEC 2
+
 static DBusGConnection *
 get_session_bus (void)
 {
@@ -160,7 +162,7 @@ can_do (const gchar *pk_action_id)
                 goto out;
         }
 
-        pk_result = polkit_context_can_caller_do_action (pk_context, pk_action, pk_caller);
+        pk_result = polkit_context_is_caller_authorized (pk_context, pk_action, pk_caller, FALSE, NULL);
 
 	switch (pk_result) {
         case POLKIT_RESULT_UNKNOWN:
@@ -190,13 +192,33 @@ out:
 gint
 can_set_system_timezone (void)
 {
-	return can_do ("org.gnome.clockapplet.mechanism.settimezone");
+	static gboolean cache = FALSE;
+	static time_t   last_refreshed = 0;
+	time_t          now;
+
+	time (&now);
+	if (ABS (now - last_refreshed) > CACHE_VALIDITY_SEC) {
+		cache = can_do ("org.gnome.clockapplet.mechanism.settimezone");
+		last_refreshed = now;
+	}
+
+	return cache;
 }
 
 gint
 can_set_system_time (void)
 {
-	return can_do ("org.gnome.clockapplet.mechanism.settime");
+	static gboolean cache = FALSE;
+	static time_t   last_refreshed = 0;
+	time_t          now;
+
+	time (&now);
+	if (ABS (now - last_refreshed) > CACHE_VALIDITY_SEC) {
+		cache = can_do ("org.gnome.clockapplet.mechanism.settime");
+		last_refreshed = now;
+	}
+
+	return cache;
 }
 
 typedef struct {
@@ -204,6 +226,7 @@ typedef struct {
         gchar *call;
 	gint64 time;
 	gchar *filename;
+        guint transient_parent_xid;
 	GFunc callback;
 	gpointer data;
 	GDestroyNotify notify;
@@ -274,7 +297,7 @@ do_auth_async (const gchar         *action,
 					      data, free_data,
 					      INT_MAX,
 					      G_TYPE_STRING, action,
-					      G_TYPE_UINT, 0,
+					      G_TYPE_UINT, data->transient_parent_xid,
 					      G_TYPE_INVALID);
 }
 
@@ -362,7 +385,8 @@ set_time_async (SetTimeCallbackData *data)
 }
 
 void
-set_system_time_async (gint64         time, 
+set_system_time_async (gint64         time,
+                       guint          transient_parent_xid,
 		       GFunc          callback, 
 		       gpointer       d, 
 		       GDestroyNotify notify)
@@ -372,11 +396,12 @@ set_system_time_async (gint64         time,
 	if (time == -1)
 		return;
 
-	data = g_new (SetTimeCallbackData, 1);
+	data = g_new0 (SetTimeCallbackData, 1);
 	data->ref_count = 1;
 	data->call = "SetTime";
 	data->time = time;
 	data->filename = NULL;
+        data->transient_parent_xid = transient_parent_xid;
 	data->callback = callback;
 	data->data = d;
 	data->notify = notify;
@@ -387,6 +412,7 @@ set_system_time_async (gint64         time,
 
 void
 set_system_timezone_async (const gchar    *filename,
+                           guint           transient_parent_xid,
 	             	   GFunc           callback, 
 		           gpointer        d, 
 		           GDestroyNotify  notify)
@@ -396,11 +422,12 @@ set_system_timezone_async (const gchar    *filename,
 	if (filename == NULL)
 		return;
 
-	data = g_new (SetTimeCallbackData, 1);
+	data = g_new0 (SetTimeCallbackData, 1);
 	data->ref_count = 1;
 	data->call = "SetTimezone";
 	data->time = -1;
 	data->filename = g_strdup (filename);
+        data->transient_parent_xid = transient_parent_xid;
 	data->callback = callback;
 	data->data = d;
 	data->notify = notify;
